package net.ihe.gazelle.proxy.action;

import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.Map;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.persistence.EntityManager;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.ihe.gazelle.proxy.dao.ProxyDAO;
import net.ihe.gazelle.proxy.model.message.AbstractMessage;
import net.ihe.gazelle.proxy.model.message.DicomMessage;
import net.ihe.gazelle.proxy.model.message.HL7Message;
import net.ihe.gazelle.proxy.model.message.HTTPMessage;
import net.ihe.gazelle.proxy.model.message.SyslogMessage;
import net.ihe.gazelle.proxy.netty.ChannelType;
import net.ihe.gazelle.proxy.util.Preferences;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.multipart.FilePart;
import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
import org.apache.commons.httpclient.methods.multipart.Part;
import org.apache.commons.httpclient.methods.multipart.PartSource;
import org.apache.commons.lang.StringEscapeUtils;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.faces.FacesMessages;
import org.jboss.seam.faces.Redirect;
import org.jboss.seam.international.StatusMessage;

@Name("messageBean")
@Scope(ScopeType.PAGE)
public class MessageBean {

	@In
	private EntityManager entityManager;

	@In(value = "#{facesContext.externalContext}")
	private ExternalContext extCtx;

	@In(value = "#{facesContext}")
	FacesContext facesContext;

	private AbstractMessage message;

	public AbstractMessage getMessage() {
		return message;
	}

	public void setMessage(AbstractMessage message) {
		this.message = message;
	}

	public String displayURLOfPage() {
		HttpServletRequest req = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext()
				.getRequest();
		return req.getRequestURL().toString();
	}

	public void retrieveMessage(String channelType) {
		Map<String, String> params = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
		String messageId = params.get("id");
		if (messageId == null || messageId.trim().length() < 1)
			return;
		try {
			int i = Integer.valueOf(messageId);
			message = ProxyDAO.getMessageByID(i, entityManager);
		} catch (Exception e) {
			message = null;
		}

		ChannelType type = ChannelType.valueOf(channelType);
		if (message.getChannelType() != type) {
			Redirect redirect = Redirect.instance();
			redirect.setParameter("id", messageId);

			String viewId = "";
			if (message instanceof HL7Message) {
				viewId = "/messages/hl7.xhtml";
			}
			if (message instanceof DicomMessage) {
				viewId = "/messages/dicom.xhtml";
			}
			if (message instanceof SyslogMessage) {
				viewId = "/messages/syslog.xhtml";
			}
			if (message instanceof HTTPMessage) {
				viewId = "/messages/http.xhtml";
			}

			redirect.setViewId(viewId);
			redirect.execute();
		}
	}

	public Boolean linkedRendered() {
		if (message instanceof HTTPMessage) {
			HTTPMessage httpMessage = (HTTPMessage) message;
			if (httpMessage.getMatchingMessage() != null) {
				return Boolean.TRUE;
			}
		}
		return Boolean.FALSE;
	}

	public String redirectToLinked() {
		if (linkedRendered()) {
			return "/messages/http.seam?id=" + linkedLabel();
		} else {
			return null;
		}
	}

	public String linkedLabel() {
		if (message instanceof HTTPMessage) {
			HTTPMessage httpMessage = (HTTPMessage) message;
			if (httpMessage.getMatchingMessage() == null) {
				return "";
			}
			if (httpMessage.getMatchingMessage().getId() == null) {
				return "";
			}
			return Integer.toString(httpMessage.getMatchingMessage().getId());
		}
		return "";
	}

	public void downloadFile(boolean inline) {
		try {
			HttpServletResponse response = (HttpServletResponse) extCtx.getResponse();
			InputStream messageStream = message.getMessageReceivedStream();
			ServletOutputStream servletOutputStream = response.getOutputStream();
			response.setContentType("text/plain");
			response.setContentLength(message.getMessageLength());
			if (!inline) {
				response.setHeader("Content-Disposition", "attachment;filename=\"" + message.getId() + "\"");
			}

			byte[] buf = new byte[8192];
			while (true) {
				int length = messageStream.read(buf);
				if (length < 0)
					break;
				servletOutputStream.write(buf, 0, length);
			}

			try {
				messageStream.close();
			} catch (IOException ignore) {
			}
			try {
				servletOutputStream.close();
			} catch (IOException ignore) {
			}

			facesContext.responseComplete();
		} catch (Exception e) {
			FacesMessages.instance().add(StatusMessage.Severity.ERROR, "Failed to download file", e);
		}
	}

	public Boolean shouldDisplayMessageContent() {
		if (message instanceof HTTPMessage) {
			HTTPMessage httpMessage = (HTTPMessage) message;
			if (httpMessage.getMessageReceived() == null)
				return false;

			String messageType = httpMessage.getMessageType();
			if (messageType == null || messageType.equals("")) {
				messageType = "application/octet-stream";
			}
			if (messageType.toLowerCase().endsWith("xml") || messageType.toLowerCase().endsWith("html")) {
				return true;
			}
		}
		if (message instanceof SyslogMessage || message instanceof HL7Message) {
			if (message.getMessageLength() > 0) {
				return true;
			}
		}
		return false;
	}

	public Boolean isDownloadable() {
		if (message == null || message.getMessageReceived() == null)
			return false;
		if (message.getMessageReceived().length > 0) {
			return true;
		}
		return false;
	}

	public String escape(String source) {
		String result = StringEscapeUtils.escapeHtml(source);
		return "<pre>" + result + "</pre>";
	}

	public void validate() {
		// First send the file to EVSClient
		HttpClient client = new HttpClient();
		PostMethod filePost = new PostMethod(Preferences.getProperty("evsclient.url") + "upload");

		// use the type as file name
		String messageType = message.getChannelType().name();

		PartSource partSource = message.getPartSource(messageType);
		Part[] parts = { new FilePart("message", partSource) };
		filePost.setRequestEntity(new MultipartRequestEntity(parts, filePost.getParams()));
		int status = -1;
		try {
			status = client.executeMethod(filePost);
		} catch (Exception e) {
			status = -1;
			e.printStackTrace();
		}
		// redirect the user to the good page
		boolean fail = false;
		if (status == HttpStatus.SC_OK) {
			try {
				String key = filePost.getResponseBodyAsString();
				String encodedKey = URLEncoder.encode(key, "UTF-8");

				HttpServletResponse response = (HttpServletResponse) extCtx.getResponse();
				String url = Preferences.getProperty("evsclient.url") + "validate.seam?file=" + encodedKey;

				response.sendRedirect(url);
			} catch (IOException e) {
				fail = true;
				e.printStackTrace();
			}
		} else {
			fail = true;
		}
	}
}
