package net.ihe.gazelle.proxy.action;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import net.ihe.gazelle.proxy.netty.ConnectionConfig;
import net.ihe.gazelle.proxy.netty.Proxy;

import org.apache.log4j.Logger;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;

@AutoCreate
@Scope(ScopeType.APPLICATION)
@Name("proxyBean")
public class ProxyBean implements ProxyLocal {

	public static Logger log = Logger.getLogger(ProxyBean.class);

	private List<Proxy<?, ?>> listOfProxies;

	private String ips;

	@Create
	public void startProxy() {
		listOfProxies = new ArrayList<Proxy<?, ?>>();

		StringBuilder sb = new StringBuilder("");
		try {
			boolean first = true;
			Enumeration<NetworkInterface> e = NetworkInterface.getNetworkInterfaces();
			while (e.hasMoreElements()) {
				NetworkInterface ni = (NetworkInterface) e.nextElement();
				Enumeration<InetAddress> e2 = ni.getInetAddresses();
				while (e2.hasMoreElements()) {
					InetAddress ip = (InetAddress) e2.nextElement();
					if (ip instanceof Inet4Address) {
						String hostName = ip.getHostName();
						if (!hostName.startsWith("localhost")) {
							if (!first)
								sb.append(", ");
							sb.append(ip.getHostAddress());
							sb.append(" (");
							sb.append(hostName);
							sb.append(")");
							first = false;
						}
					}
				}
			}
		} catch (Throwable e) {
			log.error(e);
		}
		ips = sb.toString();
	}

	public List<Proxy<?, ?>> getListOfProxies() {
		return listOfProxies;
	}

	@Destroy
	public void stopProxy() {
		List<Integer> ports = new ArrayList<Integer>();
		for (Proxy<?, ?> proxy : listOfProxies) {
			ports.add(new Integer(proxy.getLocalPort()));
		}

		for (Integer port : ports) {
			stopChannel(port);
		}
	}

	public Proxy<?, ?> startChannel(ConnectionConfig<?, ?> connectionConfig) {
		Proxy<?, ?> proxy = connectionConfig.getChannelType().getProxy(connectionConfig);
		proxy.start();
		listOfProxies.add(proxy);
		return proxy;
	}

	public void stopChannel(Integer localPort) {
		Proxy<?, ?> proxyChannel = findProxyWithLocalPort(localPort);
		if (proxyChannel != null) {
			proxyChannel.stop();
			listOfProxies.remove(proxyChannel);
		}
	}

	private Proxy<?, ?> findProxyWithLocalPort(int localPort) {
		for (Proxy<?, ?> proxyChannel : listOfProxies) {
			if (proxyChannel.getLocalPort() == localPort)
				return proxyChannel;
		}
		return null;
	}

	public String getIps() {
		return ips;
	}

	public boolean channelExists(int localPort) {
		Proxy<?, ?> findProxyWithLocalPort = findProxyWithLocalPort(localPort);
		return findProxyWithLocalPort != null;
	}

}
