package net.ihe.gazelle.proxy.ws;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.ejb.Stateless;
import javax.jws.HandlerChain;
import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.persistence.EntityManager;

import net.ihe.gazelle.proxy.action.ProxyLocal;
import net.ihe.gazelle.proxy.dao.ProxyDAO;
import net.ihe.gazelle.proxy.model.tm.Configuration;
import net.ihe.gazelle.proxy.model.tm.Step;
import net.ihe.gazelle.proxy.model.tm.TestInstance;
import net.ihe.gazelle.proxy.util.HibernateUtilProxy;

import org.jboss.seam.Component;

@WebService
@Stateless
@HandlerChain(file = "handlers.xml")
public class ProxyForTM implements IProxyForTM {

	public ProxyForTM() {
		super();
	}

	@WebMethod
	public void startAllChannels(List<Configuration> configurations) {
		for (Configuration configuration : configurations) {
			configuration.setTestInstance(null);
			startChannel(configuration);
		}
	}

	@WebMethod
	public void startTestInstance(TestInstance testInstance) {
		Calendar endTI = Calendar.getInstance();
		testInstance.setDate(endTI.getTime());

		endTI.add(Calendar.YEAR, 1);

		List<Step> steps = testInstance.getSteps();
		List<Configuration> configurations = testInstance.getConfigurations();

		for (Step step : steps) {
			// never stopped
			step.setDate(endTI.getTime());
			step.setTestInstance(testInstance);
			step.setMessage(null);
			List<Integer> ids = step.getReceiverIds();
			List<Configuration> stepConfigurations = new ArrayList<Configuration>();
			if (ids != null) {
				stepConfigurations = getStepConfigurations(configurations, ids);
			}
			step.setReceivers(stepConfigurations);

			ids = step.getSenderIds();
			stepConfigurations = new ArrayList<Configuration>();
			if (ids != null) {
				stepConfigurations = getStepConfigurations(configurations, ids);
			}
			step.setSenders(stepConfigurations);
		}

		for (Configuration configuration : configurations) {
			configuration.setTestInstance(testInstance);
			startChannel(configuration);
		}

		EntityManager entityManager = HibernateUtilProxy.buildEntityManager();
		entityManager.persist(testInstance);
		HibernateUtilProxy.releaseEntityManager(entityManager);

	}

	private List<Configuration> getStepConfigurations(List<Configuration> configurations, List<Integer> ids) {
		List<Configuration> stepConfigurations = new ArrayList<Configuration>();
		for (Integer id : ids) {
			Configuration stepConfiguration = getConfigurationByTmId(id, configurations);
			if (stepConfiguration != null && stepConfiguration.getProxyPort() > 0) {
				stepConfigurations.add(stepConfiguration);
			}
		}
		return stepConfigurations;
	}

	private void startChannel(Configuration configuration) {
		if (configuration.getProxyPort() > 0 && configuration.getProxyPort() < 65536 && configuration.getPort() > 0
				&& configuration.getPort() < 65536) {
			ProxyLocal proxyBean = (ProxyLocal) Component.getInstance("proxyBean");
			if (!proxyBean.channelExists(configuration.getProxyPort())) {
				proxyBean.startChannel(new TMConfiguration(configuration));
			}
		}
	}

	private Configuration getConfigurationByTmId(int id, List<Configuration> configurations) {
		for (Configuration configuration : configurations) {
			if (configuration.getTmId() == id) {
				return configuration;
			}
		}
		return null;
	}

	@WebMethod
	public void markTestStep(int testStepId) {
		ProxyDAO.updateStepDate(testStepId);
	}

}
