package net.ihe.gazelle.proxy.dao;

import java.util.Date;

import net.ihe.gazelle.common.filter.hql.HQLQueryBuilder;
import net.ihe.gazelle.common.filter.hql.HQLRestrictions;
import net.ihe.gazelle.proxy.model.message.AbstractMessage;
import net.ihe.gazelle.proxy.netty.ChannelType;
import net.ihe.gazelle.proxy.netty.channel.ProxySide;

import org.apache.commons.lang.StringUtils;

public class MessageFilterStandard<T extends AbstractMessage> implements MessageFilter<T> {

	private ChannelType messageType = ChannelType.HTTP;

	private String initiatorIP;
	private Integer initiatorPort;

	private String responderIP;
	private Integer responderPort;

	private Integer proxyPort;

	private Date dateFrom;
	private Date dateTo;

	private Integer connectionId;

	private ProxySide proxySide = null;

	private String dicomAffectedSopClassUID;
	private String dicomRequestedSopClassUID;
	private String dicomCommandField;

	public String getInitiatorIP() {
		return initiatorIP;
	}

	public void setInitiatorIP(String initiatorIP) {
		this.initiatorIP = initiatorIP;
	}

	public Integer getInitiatorPort() {
		return initiatorPort;
	}

	public void setInitiatorPort(Integer initiatorPort) {
		this.initiatorPort = initiatorPort;
	}

	public String getResponderIP() {
		return responderIP;
	}

	public void setResponderIP(String responderIP) {
		this.responderIP = responderIP;
	}

	public Integer getResponderPort() {
		return responderPort;
	}

	public void setResponderPort(Integer responderPort) {
		this.responderPort = responderPort;
	}

	public Integer getProxyPort() {
		return proxyPort;
	}

	public void setProxyPort(Integer proxyPort) {
		this.proxyPort = proxyPort;
	}

	public ProxySide getProxySide() {
		return proxySide;
	}

	public void setProxySide(ProxySide proxySide) {
		this.proxySide = proxySide;
	}

	public Date getDateFrom() {
		return dateFrom;
	}

	public void setDateFrom(Date dateFrom) {
		this.dateFrom = dateFrom;
	}

	public Date getDateTo() {
		return dateTo;
	}

	public void setDateTo(Date dateTo) {
		this.dateTo = dateTo;
	}

	public ChannelType getMessageType() {
		return messageType;
	}

	public void setMessageType(ChannelType messageType) {
		this.messageType = messageType;
	}

	public Integer getConnectionId() {
		return connectionId;
	}

	public void setConnectionId(Integer connectionId) {
		this.connectionId = connectionId;
	}

	public String getDicomAffectedSopClassUID() {
		return dicomAffectedSopClassUID;
	}

	public void setDicomAffectedSopClassUID(String dicomAffectedSopClassUID) {
		this.dicomAffectedSopClassUID = dicomAffectedSopClassUID;
	}

	public String getDicomRequestedSopClassUID() {
		return dicomRequestedSopClassUID;
	}

	public void setDicomRequestedSopClassUID(String dicomRequestedSopClassUID) {
		this.dicomRequestedSopClassUID = dicomRequestedSopClassUID;
	}

	public String getDicomCommandField() {
		return dicomCommandField;
	}

	public void setDicomCommandField(String dicomCommandField) {
		this.dicomCommandField = dicomCommandField;
	}

	public void reset() {
		messageType = ChannelType.HTTP;
		initiatorIP = null;
		initiatorPort = null;
		responderIP = null;
		responderPort = null;
		proxyPort = null;
		dateFrom = null;
		dateTo = null;
		connectionId = null;
		proxySide = null;

		dicomAffectedSopClassUID = null;
		dicomRequestedSopClassUID = null;
		dicomCommandField = null;
	}

	public void appendFilters(HQLQueryBuilder<T> criteria) {
		criteria.addRestriction(HQLRestrictions.eq("class", getMessageType().getDiscriminator()));

		addEq(criteria, "fromIP", StringUtils.trimToNull(initiatorIP));
		addEq(criteria, "localPort", initiatorPort);

		addEq(criteria, "proxyPort", proxyPort);

		addEq(criteria, "toIP", StringUtils.trimToNull(responderIP));
		addEq(criteria, "remotePort", responderPort);

		addEq(criteria, "connectionId", connectionId);
		addEq(criteria, "proxySide", proxySide);

		if (dateFrom != null) {
			criteria.addRestriction(HQLRestrictions.ge("dateReceived", dateFrom));
		}
		if (dateTo != null) {
			criteria.addRestriction(HQLRestrictions.le("dateReceived", dateTo));
		}

		if (messageType == ChannelType.DICOM) {
			addEq(criteria, "affectedSopClassUID", dicomAffectedSopClassUID);
			addEq(criteria, "requestedSopClassUID", dicomRequestedSopClassUID);
			addEq(criteria, "commandField", dicomCommandField);
		}
	}

	private void addEq(HQLQueryBuilder<T> criteria, String parameterName, Object value) {
		if (value != null) {
			criteria.addEq(parameterName, value);
		}
	}

}
