package net.ihe.gazelle.proxy.listeners;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;

import javax.persistence.EntityManager;

import jp.digitalsensation.ihej.transactionmonitor.dicom.messageexchange.DimseMessage;
import net.ihe.gazelle.proxy.model.message.DicomMessage;
import net.ihe.gazelle.proxy.netty.channel.ProxySide;
import net.ihe.gazelle.proxy.util.Preferences;

import org.apache.commons.io.FileUtils;

public class DicomEventListener extends SameEventListener<DimseMessage> {

	protected static final Charset UTF_8 = Charset.forName("UTF-8");

	public DicomEventListener(MessageListener messageListener) {
		super(messageListener);
	}

	protected void saveMessage(DimseMessage dicom, String requesterIp, int requesterPort, int proxyPort,
			String responderIp, int responderPort, int channelId, ProxySide side) {
		// Messages are coming from different threads!
		synchronized (this) {
			EntityManager em = messageListener.getEntityManager();

			DicomMessage messageToStore = new DicomMessage(requesterIp, requesterPort, proxyPort, responderIp,
					responderPort, channelId, side);
			if (dicom.getDataSet() != null) {
				try {
					String storageDicom = Preferences.getProperty("storage.dicom");
					File storageDicomFolder = new File(storageDicom);

					String fileName = Long.toHexString(System.nanoTime());
					if (fileName.length() < 3) {
						fileName = "000" + fileName;
					}

					int l = fileName.length();
					File targetFolder1 = new File(storageDicomFolder, fileName.substring(l - 3, l - 2));
					File targetFolder2 = new File(targetFolder1, fileName.substring(l - 2, l - 1));
					File targetFolder3 = new File(targetFolder2, fileName.substring(l - 1, l));

					targetFolder3.mkdirs();

					File targetFile = new File(targetFolder3, fileName);

					File sourceFile = new File(dicom.getDataSet().getCanonicalPath());

					FileUtils.copyFile(sourceFile, targetFile);

					String pathToStore = targetFile.getAbsolutePath();
					pathToStore = pathToStore.replace(storageDicom, "");

					messageToStore.setMessageReceived(pathToStore.getBytes(UTF_8));

					sourceFile.delete();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if (em != null && dicom.getCommandSet() != null && dicom.getCommandSet().length > 0)
				messageToStore.setFileCommandSet(em, dicom.getCommandSet());
			messageToStore.setFileTransfertSyntax(dicom.getTransferSyntax());

			messageListener.processMessage(em, messageToStore);
		}
	}

}
