package net.ihe.gazelle.proxy.listeners;

import java.nio.charset.Charset;
import java.util.Collections;
import java.util.Map;
import java.util.Set;
import java.util.WeakHashMap;

import javax.persistence.EntityManager;

import net.ihe.gazelle.proxy.model.message.HTTPMessage;
import net.ihe.gazelle.proxy.netty.channel.ProxySide;
import net.ihe.gazelle.proxy.netty.channel.ProxyTools;

import org.apache.log4j.Logger;
import org.jboss.netty.handler.codec.http.HttpMessage;

public class HttpEventListener extends SameEventListener<HttpMessage> {

	private static final Charset UTF_8 = Charset.forName("UTF-8");
	public static Logger log = Logger.getLogger(HttpEventListener.class);

	private Map<Integer, HTTPMessage> pendingRequest = Collections
			.synchronizedMap(new WeakHashMap<Integer, HTTPMessage>());

	public HttpEventListener(MessageListener messageListener) {
		super(messageListener);
	}

	public void saveMessage(final HttpMessage message, final String requesterIp, final int requesterPort,
			final int proxyPort, final String responderIp, final int responderPort, final int channelId,
			final ProxySide side) {

		// Messages are coming from different threads!
		synchronized (this) {
			HTTPMessage soapMessage = new HTTPMessage(requesterIp, requesterPort, proxyPort, responderIp,
					responderPort, channelId, side);

			HTTPMessage matchingMessage = null;
			if (side == ProxySide.REQUEST) {
				pendingRequest.put(new Integer(channelId), soapMessage);
			} else {
				matchingMessage = pendingRequest.get(new Integer(channelId));
				if (matchingMessage != null) {
					soapMessage.setMatchingMessage(matchingMessage);
					pendingRequest.remove(new Integer(channelId));
				}
			}

			soapMessage.setHeaders(message.toString().getBytes(UTF_8));

			if (message.getContent().readable() && message.getContent().readableBytes() > 0) {
				extractContent(message, soapMessage);
			} else {
				soapMessage.setMessageReceived(new byte[0]);
			}

			EntityManager entityManager = messageListener.getEntityManager();
			messageListener.processMessage(entityManager, soapMessage);
			if (matchingMessage != null) {
				matchingMessage.setMatchingMessage(soapMessage);
				messageListener.processMessage(entityManager, matchingMessage);
			}

		}
	}

	private void extractContent(HttpMessage message, HTTPMessage soapMessage) {
		String mediaTypeString = null;

		Set<String> headerNames = message.getHeaderNames();
		for (String headerName : headerNames) {
			if (headerName.toLowerCase().equals("content-type")) {
				String header = message.getHeader(headerName);
				int semicol = header.indexOf(';');
				if (semicol >= 0) {
					header = header.substring(0, semicol);
				}
				mediaTypeString = header;
			}
		}

		soapMessage.setMessageType(mediaTypeString);
		soapMessage.setMessageReceived(ProxyTools.getBytes(message.getContent().copy()));
	}

}
