/*
 * Copyright 2009 IHE International (http://www.ihe.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.ihe.gazelle.proxy.model;

import java.util.List;

import javax.persistence.Column;
import javax.persistence.EntityManager;
import javax.persistence.MappedSuperclass;
import javax.persistence.Query;

/**
 * <b>Class Description : </b>LabelKeywordDescriptionClass<br>
 * <br>
 * This abstract class is intended to be extended by others classes, for the
 * purpose of implementing a trio of label keyword and description. T
 * 
 * @class LabelKeywordDescriptionClass.java
 * @package net.ihe.gazelle.common.audit
 * @author Jean-Baptiste Meyer / INRIA Rennes IHE development Project
 * @see > jmeyer@irisa.fr - http://www.ihe-europe.org
 * @version 1.0 - 2009, jun 08
 * 
 */

@MappedSuperclass
public abstract class LabelKeywordDescriptionClass {

	@Column(name = "keyword")
	private String keyword;

	@Column(name = "value_of_key")
	private String value;

	@Column(name = "description")
	private String description;

	public String getKeyword() {
		return keyword;
	}

	public void setKeyword(String keyword) {
		this.keyword = keyword;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public LabelKeywordDescriptionClass() {

	}

	public LabelKeywordDescriptionClass(String keyword, String labelToDisplay, String description) {
		this.description = description;
		this.value = labelToDisplay;
		this.keyword = keyword;
	}

	public LabelKeywordDescriptionClass(LabelKeywordDescriptionClass inLabelKeywordDescriptionClass) {
		if (inLabelKeywordDescriptionClass.getDescription() != null)
			this.description = inLabelKeywordDescriptionClass.getDescription();
		if (inLabelKeywordDescriptionClass.getKeyword() != null)
			this.keyword = inLabelKeywordDescriptionClass.getKeyword();
		if (inLabelKeywordDescriptionClass.getValue() != null)
			this.value = inLabelKeywordDescriptionClass.getValue();
	}

	@SuppressWarnings("unchecked")
	public static List<? extends LabelKeywordDescriptionClass> getListStatus(
			Class<? extends LabelKeywordDescriptionClass> c, EntityManager em) {

		Query query = em.createQuery("FROM " + c.getCanonicalName() + " clazz  order by clazz.id");

		return (List<LabelKeywordDescriptionClass>) query.getResultList();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((description == null) ? 0 : description.hashCode());
		result = prime * result + ((keyword == null) ? 0 : keyword.hashCode());
		result = prime * result + ((value == null) ? 0 : value.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		LabelKeywordDescriptionClass other = (LabelKeywordDescriptionClass) obj;
		if (description == null) {
			if (other.description != null)
				return false;
		} else if (!description.equals(other.description))
			return false;
		if (keyword == null) {
			if (other.keyword != null)
				return false;
		} else if (!keyword.equals(other.keyword))
			return false;
		if (value == null) {
			if (other.value != null)
				return false;
		} else if (!value.equals(other.value))
			return false;
		return true;
	}

}
