package net.ihe.gazelle.proxy.util;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;

import org.hibernate.Session;
import org.hibernate.ejb.Ejb3Configuration;

public class HibernateUtilProxy {

	private static EntityManagerFactory entityManagerFactory;
	private static final Object emfSync = new Object();

	public static EntityManager buildEntityManager() {
		buildEntityManagerFactory();

		EntityManager entityManager = entityManagerFactory.createEntityManager();
		entityManager.getTransaction().begin();
		return entityManager;
	}

	private static void buildEntityManagerFactory() {
		if (entityManagerFactory == null) {
			synchronized (emfSync) {
				if (entityManagerFactory == null) {
					Ejb3Configuration cfg = new Ejb3Configuration();
					cfg.configure("META-INF/hibernate.cfg.xml");
					entityManagerFactory = cfg.buildEntityManagerFactory();
				}
			}
		}
	}

	public static void releaseEntityManager(EntityManager entityManager) {
		entityManager.getTransaction().commit();
		entityManager.close();
	}

	public static void releaseEntityManagerFactory() {
		if (entityManagerFactory != null) {
			synchronized (emfSync) {
				if (entityManagerFactory != null) {
					entityManagerFactory.close();
					entityManagerFactory = null;
				}
			}
		}
	}

	public static Session getSession(EntityManager entityManager) {
		if (entityManager != null) {
			Object delegate = entityManager.getDelegate();
			if (delegate instanceof Session) {
				Session session = (Session) delegate;
				return session;
			}
		}
		return null;
	}

	public static EntityManagerFactory getEntityManagerFactory() {
		buildEntityManagerFactory();
		return entityManagerFactory;
	}

}
