package net.ihe.gazelle.proxy.netty.channel;

import org.apache.log4j.Logger;
import org.jboss.netty.buffer.ChannelBuffer;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.channel.ChannelStateEvent;
import org.jboss.netty.channel.ExceptionEvent;
import org.jboss.netty.channel.MessageEvent;
import org.jboss.netty.channel.SimpleChannelUpstreamHandler;

/**
 * The Class ProxyHandler.<br />
 * Two ProxyHandlers exchange their messages to each side.
 */
public abstract class ProxyHandler extends SimpleChannelUpstreamHandler {

	private static Logger log = Logger.getLogger(ProxyHandler.class);

	protected Channel proxiedChannel;

	protected Channel requestChannel;

	public Channel getRequestChannel() {
		return requestChannel;
	}

	public void setRequestChannel(Channel requestChannel) {
		this.requestChannel = requestChannel;
	}

	@Override
	public void messageReceived(ChannelHandlerContext ctx, final MessageEvent e) throws Exception {
		ChannelBuffer msg = (ChannelBuffer) e.getMessage();
		synchronized (requestChannel) {
			proxiedChannel.write(msg.copy());
			// If proxiedChannel is saturated, do not read until notified in
			// other ProxyHandler.channelInterestChanged().
			if (!proxiedChannel.isWritable()) {
				e.getChannel().setReadable(false);
			}
		}

		ctx.sendUpstream(e);
	}

	@Override
	public void channelInterestChanged(ChannelHandlerContext ctx, ChannelStateEvent e) throws Exception {
		// If outboundChannel is not saturated anymore, continue accepting
		// the incoming traffic from the inboundChannel.
		synchronized (requestChannel) {
			if (e.getChannel().isWritable()) {
				proxiedChannel.setReadable(true);
			}
		}
	}

	@Override
	public void channelClosed(ChannelHandlerContext ctx, ChannelStateEvent e) throws Exception {
		if (proxiedChannel != null) {
			ProxyTools.closeOnFlush(proxiedChannel);
		}
	}

	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, ExceptionEvent e) throws Exception {
		log.error("exceptionCaught", e.getCause());
		ProxyTools.closeOnFlush(e.getChannel());
	}

}
