package net.ihe.gazelle.proxy.netty.channel;

import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.util.List;

import net.ihe.gazelle.proxy.netty.Proxy;

import org.jboss.netty.bootstrap.ClientBootstrap;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelFuture;
import org.jboss.netty.channel.ChannelFutureListener;
import org.jboss.netty.channel.ChannelHandler;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.channel.ChannelPipeline;
import org.jboss.netty.channel.ChannelStateEvent;

public class ProxyInboundHandler<REQU, RESP> extends ProxyHandler {

	private Proxy<REQU, RESP> proxy;

	public ProxyInboundHandler(Proxy<REQU, RESP> proxy) {
		super();
		this.proxy = proxy;
	}

	@Override
	public void channelOpen(ChannelHandlerContext ctx, ChannelStateEvent e) throws Exception {
		final Channel inboundChannel = e.getChannel();

		// Suspend incoming traffic until connected to the remote host.
		inboundChannel.setReadable(false);
		requestChannel = inboundChannel;

		// Start the connection attempt.
		ClientBootstrap cb = new ClientBootstrap(Proxy.clientChannelFactory);
		ChannelPipeline p = cb.getPipeline();

		int i = 0;
		List<ChannelHandler> connectionHandlers = proxy.getResponseConnectionHandlers();
		if (connectionHandlers != null) {
			for (ChannelHandler channelHandler : connectionHandlers) {
				p.addLast("handler" + i, channelHandler);
				i++;
			}
		}

		ProxyOutboundHandler outboundHandler = new ProxyOutboundHandler(inboundChannel, this);
		p.addLast("handler", outboundHandler);

		List<ChannelHandler> decoderChannels = proxy.getResponseDecoderChannels();
		for (ChannelHandler channelHandler : decoderChannels) {
			p.addLast("handler" + i, channelHandler);
			i++;
		}

		SocketAddress remoteAddress = inboundChannel.getRemoteAddress();
		String fromIp = "";
		int fromPort = 0;
		if (remoteAddress instanceof InetSocketAddress) {
			InetSocketAddress inetSocketAddress = (InetSocketAddress) remoteAddress;
			fromIp = inetSocketAddress.getHostName();
			fromPort = inetSocketAddress.getPort();
		}

		// send the response to the listener
		p.addLast("logger", new ProxyLoggerResponse<REQU, RESP>(proxy, outboundHandler, fromIp, fromPort));

		// Connection to real server
		ChannelFuture f = cb.connect(new InetSocketAddress(proxy.getRemoteHost(), proxy.getRemotePort()));
		proxiedChannel = f.getChannel();

		f.addListener(new ChannelFutureListener() {
			public void operationComplete(ChannelFuture future) throws Exception {
				if (!future.isSuccess()) {
					// Close the connection if the connection attempt has
					// failed.
					inboundChannel.close();
				}
			}
		});

	}

	public void onChannelOpen() {
		requestChannel.setReadable(true);
	}

}
